@@ -17,7 +17,7 @@ from .constants import HOMEDIR, KEEP
 from .puddleobjects import (
     get_icon, gettaglist, partial,
     settaglist, winsettings, ListButtons, MoveButtons, OKCancel,
-    PicWidget, PuddleConfig, natsort_case_key)
+    PicWidget, PuddleConfig, natural_sort_key)
 from .translations import translate
 from .util import pprint_tag
 from .util import to_string
@@ -313,12 +313,13 @@ class ImportTextFile(QDialog):
             f = open(filename, 'r')
         except (IOError, OSError) as detail:
             errormsg = translate('Text File -> Tag',
-                                 "The file <b>%1</b> couldn't be loaded.<br /> "
-                                 "Do you want to choose another?")
+                                 "The file <b>{}</b> couldn't be loaded.<br /> "
+                                 "Do you want to choose another?"
+                                 ).format(filename)
 
             ret = QMessageBox.question(self,
                                        translate('Text File -> Tag', "Error"),
-                                       translate('Text File -> Tag', errormsg.arg(filename)))
+                                       errormsg)
 
             if ret == QMessageBox.StandardButton.Yes:
                 return self.openFile()
@@ -455,20 +456,22 @@ class StatusWidgetItem(QTableWidgetItem):
         self._original = (text, preview, status)
         self.linked = []
 
-    def _get_preview(self):
+    @property
+    def preview(self):
         return self.font().bold()
 
-    def _set_preview(self, value):
+    @preview.setter
+    def preview(self, value):
         font = self.font()
         font.setBold(value)
         self.setFont(font)
 
-    preview = property(_get_preview, _set_preview)
-
-    def _get_status(self):
+    @property
+    def status(self):
         return self._status
 
-    def _set_status(self, status):
+    @status.setter
+    def status(self, status):
         if status and status in self.statusColors:
             if self._status == ADD and status != REMOVE:
                 return
@@ -478,8 +481,6 @@ class StatusWidgetItem(QTableWidgetItem):
             self.setBackground(QTableWidgetItem().background())
             self._status = None
 
-    status = property(_get_status, _set_status)
-
     def __lt__(self, item):
         if self.text().upper() < item.text().upper():
             return True
@@ -518,7 +519,7 @@ class StatusWidgetCombo(QComboBox):
         self.setSizeAdjustPolicy(QComboBox.SizeAdjustPolicy.AdjustToContents)
 
         items = map(str, items)
-        items = sorted(items, key=natsort_case_key)
+        items = sorted(items, key=natural_sort_key)
         if len(items) > 1:
             items.append(r'\\'.join(items))
         self.addItem(KEEP)
@@ -534,22 +535,20 @@ class StatusWidgetCombo(QComboBox):
         self._original = (items, preview, status)
         self.linked = []
 
-    def _get_preview(self):
+    @property
+    def preview(self):
         return False
-        return self.font().bold()
 
-    def _set_preview(self, value):
+    @preview.setter
+    def preview(self, _):
         return
-        font = self.font()
-        font.setBold(value)
-        self.setFont(font)
 
-    preview = property(_get_preview, _set_preview)
-
-    def _get_status(self):
+    @property
+    def status(self):
         return self._status
 
-    def _set_status(self, status):
+    @status.setter
+    def status(self, status):
         if status and status in self.statusColors:
             if self._status == ADD and status != REMOVE:
                 return
@@ -559,14 +558,12 @@ class StatusWidgetCombo(QComboBox):
             self.setBackground(None)
             self._status = None
 
-    status = property(_get_status, _set_status)
-
     def setBackground(self, brush=None):
         if brush is None:
             color = QLineEdit().palette().color(QPalette.ColorRole.Base).name()
         else:
             color = brush.color().name()
-        self.setStyleSheet("QComboBox { background-color: %s; }" % color);
+        self.setStyleSheet("QComboBox { background-color: %s; }" % color)
 
     def background(self):
         brush = QBrush()
@@ -660,7 +657,7 @@ class ExTags(QDialog):
         self._reset = QToolButton()
         self._reset.setToolTip(translate('Extended Tags',
                                          'Resets the selected fields to their original value.'))
-        self._reset.setIcon(get_icon('edit-undo', ':/undo.png'))
+        self._reset.setIcon(get_icon('edit-undo'))
         self._reset.clicked.connect(self.resetFields)
 
         self.listbuttons = ListButtons()
