@@ -16,7 +16,7 @@ from .. import musiclib, about as about
 from ..util import split_by_tag, translate, to_string
 from .. import functions
 from .tagtools import *
-from ..constants import HOMEDIR
+from ..constants import HOMEDIR, SEPARATOR
 
 path = os.path
 
@@ -455,6 +455,8 @@ def run_func(selectedfiles, func):
             for field in fields:
                 val = function(rowtags.get(field, ''), rowtags, state, r_tags=f)
                 if val is not None:
+                    if isinstance(val, str) and SEPARATOR in val:
+                        val = val.split(SEPARATOR)
                     if hasattr(val, 'items'):
                         ret.update(val)
                     else:
@@ -548,30 +550,30 @@ def update_status(enable=True):
     x = findfunc.filenametotag(pattern, tag[PATH], True)
     emit('ftstatus', display_tag(x))
 
-    bold_error = translate("Status Bar", "<b>%s</b>")
+    bold_error = translate("Status Bar", "<b>{}</b>")
 
     try:
         newfilename = functions.move(tag, pattern, tag, state=state.copy())
         if newfilename:
             newfilename = newfilename['__path']
-            emit('tfstatus', translate("Status Bar",
-                                       "New Filename: <b>%1</b>").arg(
-                decode_fn(newfilename)))
+            emit('tfstatus', translate('Status Bar',
+                                       "New Filename: <b>{}</b>"
+                                       ).format(decode_fn(newfilename)))
         else:
             emit('tfstatus', '<b>No change</b>')
     except findfunc.ParseError as e:
-        emit('tfstatus', bold_error % e.message)
+        emit('tfstatus', bold_error.format(e.message))
 
     try:
         newfolder = functions.tag_dir(tag.tags, pattern, tag, state)
         if newfolder:
             newfolder = newfolder['__dirpath']
-            dirstatus = translate("Dir Renaming",
-                                  "Rename: <b>%1</b> to: <i>%2</i>")
-            dirstatus = dirstatus.arg(tag[DIRPATH]).arg(decode_fn(newfolder))
+            dirstatus = translate('Dir Renaming',
+                                  "Rename: <b>{}</b> to: <i>{}</i>"
+                                  ).format(tag[DIRPATH], decode_fn(newfolder))
             emit('renamedirstatus', dirstatus)
     except findfunc.ParseError as e:
-        emit('renamedirstatus', bold_error % e.message)
+        emit('renamedirstatus', bold_error.format(e.message))
 
     selected = status['selectedtags']
     if not selected:
@@ -595,7 +597,7 @@ class _SignalObject(QObject):
     renamedirs = pyqtSignal(list, name='renamedirs')
     onetomany = pyqtSignal(dict, name='onetomany')
     renameselected = pyqtSignal(name='renameselected')
-    adddock = pyqtSignal(str, 'QDialog', int, name='adddock')
+    adddock = pyqtSignal(str, object, int, name='adddock')
     highlight = pyqtSignal(list, name='highlight')
     writeaction = pyqtSignal([object, object, dict], name='writeaction')
 
