@@ -11,26 +11,29 @@ import sys
 import time
 from bisect import bisect_left, insort_left  # for unique function.
 from collections import defaultdict
+from collections.abc import Callable, Generator
 from copy import copy
 from functools import partial
 from glob import glob
 from io import StringIO
 from itertools import groupby  # for unique function.
+from typing import Any, Generator, List, Optional, Tuple, Union
 
-from PyQt5.QtCore import QBuffer, QByteArray, QDir, QRectF, QSettings, QSize, QThread, QTimer, Qt, pyqtSignal
+from PyQt5.QtCore import QBuffer, QByteArray, QCollator, QCollatorSortKey, QDir, QLocale, QObject, QRectF, QSettings, \
+    QSize, QThread, QTimer, Qt, pyqtSignal
 from PyQt5.QtCore import QFile, QIODevice
 from PyQt5.QtGui import QIcon, QBrush, QPixmap, QImage, \
     QKeySequence
 from PyQt5.QtSvg import QGraphicsSvgItem, QSvgRenderer
-from PyQt5.QtWidgets import QAction, QApplication, QComboBox, QDesktopWidget, QDialog, QDialogButtonBox, \
+from PyQt5.QtWidgets import QAbstractItemView, QAction, QApplication, QComboBox, QDialog, QDialogButtonBox, \
     QDockWidget, QFileDialog, QFrame, QGraphicsPixmapItem, QGraphicsScene, QGraphicsView, QGridLayout, QHBoxLayout, \
-    QHeaderView, QLabel, QLineEdit, QListWidget, QMenu, QMessageBox, QProgressBar, QPushButton, QSizePolicy, \
+    QHeaderView, QLabel, QLayout, QLineEdit, QListWidget, QMenu, QMessageBox, QProgressBar, QPushButton, QSizePolicy, \
     QTextEdit, QToolButton, QVBoxLayout, QWidget
 from configobj import ConfigObjError
 
 from . import audioinfo
 from .audioinfo import (IMAGETYPES, DESCRIPTION, DATA, IMAGETYPE, DEFAULT_COVER,
-                        INFOTAGS)
+                        INFOTAGS, get_mime)
 from .constants import ACTIONDIR, SAVEDIR, CONFIGDIR
 from .translations import translate
 
@@ -50,66 +53,37 @@ SD_PATTERNS = [
 ]
 
 mod_keys = {
-    Qt.ShiftModifier: 'Shift',
-    Qt.MetaModifier: 'Meta',
-    Qt.AltModifier: 'Alt',
-    Qt.ControlModifier: 'Ctrl',
-    Qt.NoModifier: '',
-    Qt.KeypadModifier: '',
-    Qt.GroupSwitchModifier: '', }
+    Qt.KeyboardModifier.ShiftModifier: 'Shift',
+    Qt.KeyboardModifier.MetaModifier: 'Meta',
+    Qt.KeyboardModifier.AltModifier: 'Alt',
+    Qt.KeyboardModifier.ControlModifier: 'Ctrl',
+    Qt.KeyboardModifier.NoModifier: '',
+    Qt.KeyboardModifier.KeypadModifier: '',
+    Qt.KeyboardModifier.GroupSwitchModifier: '', }
 
 
 def keycmp(modifier):
-    if modifier == Qt.CTRL:
+    if modifier == Qt.Modifier.CTRL:
         return 4
-    elif modifier == Qt.SHIFT:
+    elif modifier == Qt.Modifier.SHIFT:
         return 3
-    elif modifier == Qt.ALT:
+    elif modifier == Qt.Modifier.ALT:
         return 2
-    elif modifier == Qt.META:
+    elif modifier == Qt.Modifier.META:
         return 1
     else:
         return 0
 
 
-try:
-    permutations = itertools.permutations
-except AttributeError:
-    # Using python < 2.6
-    def permutations(iterable, r=None):
-        # permutations('ABCD', 2) --> AB AC AD BA BC BD CA CB CD DA DB DC
-        # permutations(range(3)) --> 012 021 102 120 201 210
-        pool = tuple(iterable)
-        n = len(pool)
-        r = n if r is None else r
-        if r > n:
-            return
-        indices = list(range(n))
-        cycles = list(range(n, n - r, -1))
-        yield tuple(pool[i] for i in indices[:r])
-        while n:
-            for i in reversed(list(range(r))):
-                cycles[i] -= 1
-                if cycles[i] == 0:
-                    indices[i:] = indices[i + 1:] + indices[i:i + 1]
-                    cycles[i] = n - i
-                else:
-                    j = cycles[i]
-                    indices[i], indices[-j] = indices[-j], indices[i]
-                    yield tuple(pool[i] for i in indices[:r])
-                    break
-            else:
-                return
-
 modifiers = {}
 for i in range(1, len(mod_keys)):
-    for keys in set(permutations(mod_keys, i)):
+    for keys in set(itertools.permutations(mod_keys, i)):
         mod = keys[0]
         for key in keys[1:]:
             mod = mod | key
         modifiers[int(mod)] = '+'.join(mod_keys[key] for key in sorted(keys, key=keycmp) if mod_keys[key])
 
-mod_keys = set((Qt.Key_Shift, Qt.Key_Control, Qt.Key_Meta, Qt.Key_Alt))
+mod_keys = set((Qt.Key.Key_Shift, Qt.Key.Key_Control, Qt.Key.Key_Meta, Qt.Key.Key_Alt))
 
 imagetypes = [
     (translate('Cover Type', 'Other'), translate("Cover Type", 'O')),
@@ -206,7 +180,7 @@ class PuddleConfig(object):
     def __init__(self, filename=None):
         if not filename:
             filename = os.path.join(CONFIGDIR, 'puddletag.conf')
-        self._setFilename(filename)
+        self.filename = filename
 
         self.setSection = self.set
         self.load = self.get
@@ -247,9 +221,12 @@ class PuddleConfig(object):
         self.data = defaultdict(lambda: {})
         if os.path.exists(self.filename):
             try:
-                self.data.update(json.loads(open(self.filename, 'r').read()))
-            except:
-                pass
+                with open(self.filename, 'r', encoding='utf-8') as config_file:
+                    self.data.update(json.load(config_file))
+            except json.JSONDecodeError as e:
+                print(f'Error parsing config file {self.filename}: {e}')
+            except Exception as e:
+                print(f'Unexpected error while reading config file {self.filename}: {e}')
 
     def save(self):
         actions = self.data.get('puddleactions')
@@ -264,24 +241,25 @@ class PuddleConfig(object):
         with open(filename, 'w') as fo:
             fo.write(json.dumps(dict(self.data), indent=2))
 
-    def _setFilename(self, filename):
+
+    @property
+    def filename(self):
+        return self._filename
+
+    @filename.setter
+    def filename(self, filename):
         logging.debug(f'reading config file {filename}')
         self._filename = filename
         self.savedir = os.path.dirname(filename)
         self.reload()
 
-    def _getFilename(self):
-        return self._filename
-
     def sections(self):
         return list(self.data.keys())
 
-    filename = property(_getFilename, _setFilename)
-
 
 def _getSettings():
     filename = os.path.join(CONFIGDIR, 'windowsizes')
-    return QSettings(filename, QSettings.IniFormat)
+    return QSettings(filename, QSettings.Format.IniFormat)
 
 
 def savewinsize(name, dialog, settings=_getSettings()):
@@ -463,15 +441,18 @@ HORIZONTAL = 1
 VERTICAL = 0
 
 
-def get_icon(name, backup):
-    if not name and not backup:
+def get_icon(name: Optional[str] = None, fallback: Optional[str] = None) -> QIcon:
+    """Return the icon with the given name from the current icon theme.
+
+    If the theme does not contain such icon, fallback to built-in png of
+    the same name. The fallback file can be overriden by providing a filename
+    as the second argument.
+    """
+    if not name and not fallback:
         return QIcon()
-    elif not name and backup:
-        return QIcon(backup)
-    try:
-        return QIcon.fromTheme(name, QIcon(backup))
-    except AttributeError:
-        return QIcon(backup)
+
+    fallback = fallback or f'{name}.png'
+    return QIcon.fromTheme(name, QIcon(f'icons:{fallback}'))
 
 
 def get_languages(dirs=None):
@@ -479,9 +460,9 @@ def get_languages(dirs=None):
     if dirs is not None:
         for d in dirs:
             files.extend(glob(os.path.join(d, "*.qm")))
-    d = QDir(':/')
-    if d.cd('translations'):
-        files.extend([os.path.join(':/translations', t) for t in
+    d = QDir('translations:./')
+    if not d.isEmpty():
+        files.extend([os.path.join('translations:./', t) for t in
                       map(str, d.entryList(['*.qm']))])
 
     ret = {}
@@ -513,16 +494,16 @@ def errormsg(parent, msg, maximum):
         False if No.
         None if just yes."""
     if maximum > 1:
-        mb = QMessageBox(QMessageBox.Warning, translate("Defaults", 'Error'),
+        mb = QMessageBox(QMessageBox.Icon.Warning, translate("Defaults", 'Error'),
                          msg + translate("Defaults", "<br /> Do you want to continue?"),
-                         QMessageBox.Yes | QMessageBox.No | QMessageBox.YesToAll,
+                         QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.YesToAll,
                          parent)
-        mb.setDefaultButton(QMessageBox.Yes)
-        mb.setEscapeButton(QMessageBox.No)
+        mb.setDefaultButton(QMessageBox.StandardButton.Yes)
+        mb.setEscapeButton(QMessageBox.StandardButton.No)
         ret = mb.exec_()
-        if ret == QMessageBox.No:
+        if ret == QMessageBox.StandardButton.No:
             return False
-        elif ret == QMessageBox.YesToAll:
+        elif ret == QMessageBox.StandardButton.YesToAll:
             return True
     else:
         singleerror(parent, msg)
@@ -619,24 +600,22 @@ def unique(seq, stable=False):
     return result
 
 
-class compare:
-    "Natural sorting class."
+def natural_sort_key(s: Union[str, List[str]], case_insensitive=True) -> QCollatorSortKey:
+    """Return a sort-key for natural sorting the given string.
 
-    def natsort_case_key(self, s):
-        "Used internally to get a tuple by which s is sorted."
-        convert = lambda text: int(text) if text.isdigit() else text.lower()
-        return [convert(c) for c in re.split('([0-9]+)', s)]
-
-
-natsort_case_key = compare().natsort_case_key
-
-
-# https://stackoverflow.com/a/16090640
-def natural_sort_key(s, _nsre=re.compile('([0-9]+)')):
+    Case-insensitive means uppercase- and lowercase-characters are treated equally.
+    Natural means sorting numbers by their numeric value, e.g. 100 comes after 99.
+    It also takes the global user preferences into account (e.g. LC_COLLATE).
+    """
     if isinstance(s, list):
-        s = s[0]
-    return [int(text) if text.isdigit() else text.lower()
-            for text in re.split(_nsre, s)]
+        # Join all elements with ascii unit separator
+        s = '\x1f'.join(s)
+
+    locale = QLocale.system().collation() if case_insensitive else QLocale.c()
+    collator = QCollator(locale)
+    collator.setCaseSensitivity(Qt.CaseSensitivity.CaseSensitive)
+    collator.setNumericMode(True)
+    return collator.sortKey(s)
 
 
 def dupes(l, method=None):
@@ -662,34 +641,29 @@ def dupes(l, method=None):
     return [z for z in groups if len(z) > 1]
 
 
-def getfiles(files, subfolders=False):
-    if isinstance(files, str):
+def getfiles(files: Union[str, List[str]], subfolders: bool = False) -> Generator[str, None, None]:
+    """For the given path(s), yield all the files.
+
+    If path does not exist, ignore it.
+    If path is a directory, yield all the files in that directory.
+    If subfolders is True, also recurse into subdirectories and yield their files.
+    """
+    if not isinstance(files, list):
         files = [files]
 
-    isdir = os.path.isdir
-    join = os.path.join
+    for file in files:
+        if not os.path.exists(file):
+            continue
+        if not os.path.isdir(file):
+            yield file
+            continue
 
-    temp = []
-
-    if not subfolders:
-        for f in files:
-            if not isdir(f):
-                yield f
-            else:
-                dirname, subs, fnames = next(os.walk(f))
-                for fname in fnames:
-                    yield join(dirname, fname)
-    else:
-        for f in files:
-            if not isdir(f):
-                yield f
-            else:
-                for dirname, subs, fnames in os.walk(f):
-                    for fname in fnames:
-                        yield join(dirname, fname)
-                    for sub in subs:
-                        for fname in getfiles(join(dirname, sub), subfolders):
-                            pass
+        for dirpath, dirnames, filenames in os.walk(file):
+            for filename in filenames:
+                yield os.path.join(dirpath, filename)
+            if not subfolders:
+                # don't recurse deeper
+                dirnames.clear()
 
 
 def gettags(files):
@@ -741,7 +715,7 @@ def translate_filename_pattern(pat):
         else:
             res = res + re.escape(c)
     # return res + '\Z(?ms)'
-    return res + '\Z'
+    return res + r'\Z'
 
 
 def fnmatch(pattern, files, matchcase=False):
@@ -806,7 +780,7 @@ def load_actions():
 
     files = glob(os.path.join(ACTIONDIR, '*.action'))
     if firstrun and not files:
-        filenames = [':/caseconversion.action', ':/standard.action']
+        filenames = ['data:./caseconversion.action', 'data:./standard.action']
         files = list(map(open_resourcefile, filenames))
         set_value('firstrun', False)
 
@@ -829,11 +803,12 @@ def load_actions():
 
 def open_resourcefile(filename):
     f = QFile(filename)
-    f.open(QIODevice.ReadOnly)
+    f.open(QIODevice.OpenModeFlag.ReadOnly)
     return StringIO(str(f.readAll().data(), encoding='utf-8'))
 
 
-def progress(func, pstring, maximum, threadfin=None):
+def progress(func: Callable[..., Generator[Optional[Tuple[str, int]], None, None]],
+             pstring: str, maximum: int, threadfin: Optional[Callable[[], None]] = None) -> Callable[..., None]:
     """To be used for functions that need a threaded progressbar.
 
     Note that this function will only (and is meant to) work on dialogs.
@@ -868,10 +843,6 @@ def progress(func, pstring, maximum, threadfin=None):
 
         parent = args[0]
 
-        if maximum > 1:
-            win = ProgressWin(parent, maximum, pstring)
-            win.show()
-
         if len(args) > 1:
             f = func(*args)
         else:
@@ -880,20 +851,26 @@ def progress(func, pstring, maximum, threadfin=None):
         if maximum == 1:
             errors = next(f)
             if errors and \
-                    not isinstance(errors, (str, str, int, int, str)):
+                    not isinstance(errors, (str, int)):
                 errormsg(parent, errors[0], 1)
             if threadfin:
                 threadfin()
             return
+        elif maximum > 1:
+            win = ProgressWin(parent, maximum, pstring)
+            win.show()
+        else:
+            return
+
         parent.showmessage = True
 
-        def threadfunc():
+        def threadfunc() -> None:
             i = 0
             err = False
             while not win.wasCanceled:
                 try:
                     temp = next(f)
-                    if isinstance(temp, (str, str)):
+                    if isinstance(temp, str):
                         thread.message.emit(temp)
                     elif isinstance(temp, int):
                         thread.set_max.emit(temp)
@@ -911,7 +888,7 @@ def progress(func, pstring, maximum, threadfin=None):
             if not err:
                 thread.win.emit(-1)
 
-        def threadexit(*args):
+        def threadexit(*args) -> None:
             if args[0] == -1:
                 win.close()
                 win.destroy()
@@ -924,7 +901,7 @@ def progress(func, pstring, maximum, threadfin=None):
                     except RuntimeError:
                         pass
                 return
-            elif isinstance(args[0], (str, str)):
+            elif isinstance(args[0], str):
                 if parent.showmessage:
                     ret = errormsg(parent, args[0], maximum)
                     if ret is True:
@@ -939,12 +916,12 @@ def progress(func, pstring, maximum, threadfin=None):
                 thread.start()
             win.setValue(win.value + 1)
 
-        def set_message(msg):
+        def set_message(msg: str) -> None:
             if msg != win.label.text():
                 win.label.setText(msg)
                 QApplication.processEvents()
 
-        def set_max(value):
+        def set_max(value: int) -> None:
             win.pbar.setMaximum(value)
 
         thread = PuddleThread(threadfunc, parent)
@@ -1130,7 +1107,7 @@ class ListBox(QListWidget):
         QListWidget.__init__(self, parent)
         self.yourlist = None
         self.editButton = None
-        self.setSelectionMode(self.ExtendedSelection)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
 
     def items(self):
         return list(map(self.item, range(self.count())))
@@ -1265,23 +1242,23 @@ class ListButtons(QVBoxLayout):
     def __init__(self, parent=None):
         QVBoxLayout.__init__(self, parent)
         self.addButton = QToolButton()
-        self.addButton.setIcon(get_icon('list-add', ':/filenew.png'))
+        self.addButton.setIcon(get_icon('list-add'))
         self.addButton.setToolTip(translate("List Buttons", 'Add'))
         self.removeButton = QToolButton()
-        self.removeButton.setIcon(get_icon('list-remove', ':/remove.png'))
+        self.removeButton.setIcon(get_icon('list-remove'))
         self.removeButton.setToolTip(translate("List Buttons", 'Remove'))
         self.removeButton.setShortcut('Delete')
         self.moveupButton = QToolButton()
-        self.moveupButton.setArrowType(Qt.UpArrow)
+        self.moveupButton.setArrowType(Qt.ArrowType.UpArrow)
         self.moveupButton.setToolTip(translate("List Buttons", 'Move Up'))
         self.movedownButton = QToolButton()
-        self.movedownButton.setArrowType(Qt.DownArrow)
+        self.movedownButton.setArrowType(Qt.ArrowType.DownArrow)
         self.movedownButton.setToolTip(translate("List Buttons", 'Move Down'))
         self.editButton = QToolButton()
-        self.editButton.setIcon(get_icon('document-edit', ':/edit.png'))
+        self.editButton.setIcon(get_icon('document-edit'))
         self.editButton.setToolTip(translate("List Buttons", 'Edit'))
         self.duplicateButton = QToolButton()
-        self.duplicateButton.setIcon(get_icon('edit-copy', ':/duplicate.png'))
+        self.duplicateButton.setIcon(get_icon('edit-copy'))
         self.duplicateButton.setToolTip(translate("List Buttons", 'Duplicate'))
         self.copyButton = QToolButton()
         self.copyButton.setToolTip(translate("List Buttons", 'Copy to clipboard'))
@@ -1365,7 +1342,12 @@ class MoveButtons(QWidget):
         self.next.clicked.connect(self.nextClicked)
         self.prev.clicked.connect(self.prevClicked)
 
-    def _setCurrentIndex(self, index):
+    @property
+    def index(self):
+        return self._currentindex
+
+    @index.setter
+    def index(self, index):
         try:
             if index >= len(self.arrayname) or index < 0:
                 return
@@ -1394,11 +1376,6 @@ class MoveButtons(QWidget):
 
         self.indexChanged.emit(index)
 
-    def _getCurrentIndex(self):
-        return self._currentindex
-
-    index = property(_getCurrentIndex, _setCurrentIndex)
-
     def nextClicked(self):
         self.index += 1
 
@@ -1421,8 +1398,8 @@ class OKCancel(QHBoxLayout):
         # self.addStretch()
         dbox = QDialogButtonBox()
 
-        self.okButton = dbox.addButton(dbox.Ok)
-        self.cancelButton = dbox.addButton(dbox.Cancel)
+        self.okButton = dbox.addButton(QDialogButtonBox.StandardButton.Ok)
+        self.cancelButton = dbox.addButton(QDialogButtonBox.StandardButton.Cancel)
         self.addStretch()
         self.addWidget(dbox)
 
@@ -1452,7 +1429,7 @@ class LongInfoMessage(QDialog):
 
         text = QTextEdit()
         text.setReadOnly(True)
-        # text.setWordWrapMode(QTextOption.NoWrap)
+        # text.setWordWrapMode(QTextOption.WrapMode.NoWrap)
         text.setHtml(html)
 
         okcancel = OKCancel()
@@ -1486,7 +1463,7 @@ class ArtworkLabel(QGraphicsView):
 
         self._svg = QGraphicsSvgItem()
         self._pixmap = QGraphicsPixmapItem()
-        self._pixmap.setTransformationMode(Qt.SmoothTransformation)
+        self._pixmap.setTransformationMode(Qt.TransformationMode.SmoothTransformation)
         self._scene = QGraphicsScene()
         self._scene.addItem(self._svg)
         self._scene.addItem(self._pixmap)
@@ -1518,7 +1495,7 @@ class ArtworkLabel(QGraphicsView):
 
     def mousePressEvent(self, event):
         super(ArtworkLabel, self).mousePressEvent(event)
-        if event.buttons() == Qt.LeftButton:
+        if event.buttons() == Qt.MouseButton.LeftButton:
             self.clicked.emit()
 
     def resizeEvent(self, event=None):
@@ -1529,7 +1506,7 @@ class ArtworkLabel(QGraphicsView):
         else:
             item = self._pixmap
         self.setSceneRect(item.boundingRect())
-        self.fitInView(item, Qt.KeepAspectRatio)
+        self.fitInView(item, Qt.AspectRatioMode.KeepAspectRatio)
 
     def setPixmap(self, pixmap, data=None):
         if isinstance(pixmap, str):
@@ -1579,10 +1556,10 @@ class PicWidget(QWidget):
                    If False, then these functions can be found by right clicking
                    on the picture."""
 
-        self._contextFormat = translate("Artwork Context", '%1/%2')
+        self._contextFormat = translate('Artwork Context', "{}/{}")
 
         QWidget.__init__(self, parent)
-        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         self.sizePolicy().setVerticalStretch(0)
         self.sizePolicy().setHorizontalStretch(3)
 
@@ -1591,19 +1568,19 @@ class PicWidget(QWidget):
         self.filePattern = 'folder.jpg'
 
         self.label = ArtworkLabel()
-        self.label.setFrameStyle(QFrame.Box)
+        self.label.setFrameStyle(QFrame.Shape.Box)
 
         self.label.setMinimumSize(200, 170)
         if buttons:
             self.label.setMaximumSize(200, 170)
         self._itags = []
 
-        self.label.setAlignment(Qt.AlignCenter)
+        self.label.setAlignment(Qt.AlignmentFlag.AlignCenter)
         self.label.newImages.connect(
             lambda filenames: self.addImages(self.loadPics(*filenames)))
 
         self._image_size = QLabel()
-        self._image_size.setAlignment(Qt.AlignHCenter)
+        self._image_size.setAlignment(Qt.AlignmentFlag.AlignHCenter)
 
         self._image_desc = QLineEdit(self)
 
@@ -1654,9 +1631,9 @@ class PicWidget(QWidget):
         self.readonly = readonly
 
         self.next = QToolButton()
-        self.next.setArrowType(Qt.RightArrow)
+        self.next.setArrowType(Qt.ArrowType.RightArrow)
         self.prev = QToolButton()
-        self.prev.setArrowType(Qt.LeftArrow)
+        self.prev.setArrowType(Qt.ArrowType.LeftArrow)
         self.next.clicked.connect(self.nextImage)
         self.prev.clicked.connect(self.prevImage)
 
@@ -1670,8 +1647,8 @@ class PicWidget(QWidget):
             movebuttons.addWidget(self._contextlabel)
             movebuttons.addStretch()
         else:
-            self.next.setArrowType(Qt.UpArrow)
-            self.prev.setArrowType(Qt.DownArrow)
+            self.next.setArrowType(Qt.ArrowType.UpArrow)
+            self.prev.setArrowType(Qt.ArrowType.DownArrow)
             movebuttons = QVBoxLayout()
             movebuttons.addStretch()
             movebuttons.addWidget(self.next)
@@ -1689,13 +1666,13 @@ class PicWidget(QWidget):
             v.addWidget(self._image_size)
             v.addStretch()
 
-        h = QHBoxLayout();
-        h.addStretch();
+        h = QHBoxLayout()
+        h.addStretch()
         h.addLayout(v)
         if not buttons:
             h.addLayout(movebuttons)
             context_box = QHBoxLayout()
-            context_box.setAlignment(Qt.AlignHCenter)
+            context_box.setAlignment(Qt.AlignmentFlag.AlignHCenter)
             context_box.addWidget(self._contextlabel)
             vbox.addLayout(context_box)
         h.addStretch()
@@ -1706,14 +1683,14 @@ class PicWidget(QWidget):
         if buttons:
             vbox.addLayout(movebuttons)
         vbox.addStretch()
-        vbox.setAlignment(Qt.AlignCenter)
+        vbox.setAlignment(Qt.AlignmentFlag.AlignCenter)
 
         self.label.clicked.connect(self.maxImage)
 
         hbox = QHBoxLayout()
         hbox.addLayout(vbox)
         hbox.addStrut(12)
-        hbox.setSizeConstraint(hbox.SetMinAndMaxSize)
+        hbox.setSizeConstraint(QLayout.SizeConstraint.SetMinAndMaxSize)
         self.setLayout(hbox)
 
         if buttons:
@@ -1723,7 +1700,7 @@ class PicWidget(QWidget):
             self.removepic = listbuttons.removeButton
             self.editpic = listbuttons.editButton
             self.savepic = QToolButton()
-            self.savepic.setIcon(QIcon(':/save.png'))
+            self.savepic.setIcon(get_icon('document-save'))
             self.savepic.setIconSize(QSize(16, 16))
             self.copypic = listbuttons.copyButton
             self.pastepic = listbuttons.pasteButton
@@ -1734,7 +1711,7 @@ class PicWidget(QWidget):
             hbox.addLayout(listbuttons)
 
         else:
-            self.label.setContextMenuPolicy(Qt.ActionsContextMenu)
+            self.label.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)
             self.savepic = QAction(translate("Artwork", "&Save cover to file"), self)
             self.label.addAction(self.savepic)
 
@@ -1780,7 +1757,12 @@ class PicWidget(QWidget):
 
         self._lastdata = None
 
-    def _setContext(self, text):
+    @property
+    def context(self):
+        return self._contextlabel.text()
+
+    @context.setter
+    def context(self, text):
         if not text:
             self._contextlabel.setVisible(False)
             self._contextlabel.setText('')
@@ -1788,11 +1770,6 @@ class PicWidget(QWidget):
             self._contextlabel.setText(translate("Artwork Context", text))
             self._contextlabel.setVisible(True)
 
-    def _getContext(self):
-        return self._contextlabel.text()
-
-    context = property(_getContext, _setContext)
-
     def setDescription(self, text):
         '''Sets the description of the current image to the text in the
             description text box.'''
@@ -1843,7 +1820,7 @@ class PicWidget(QWidget):
         if not image.isNull():
             ba = QByteArray()
             data = QBuffer(ba)
-            data.open(QIODevice.WriteOnly)
+            data.open(QIODevice.OpenModeFlag.WriteOnly)
             # TODO: Don't transform to JPG
             image.save(data, "JPG")
             data = bytes(data.data())
@@ -1852,7 +1829,7 @@ class PicWidget(QWidget):
                 "height": image.height(),
                 "width": image.width(),
                 "size": len(data),
-                "mime": "image/jpeg",
+                "mime": get_mime(data),
                 "description": "",
                 "imagetype": 3
             }
@@ -1904,10 +1881,14 @@ class PicWidget(QWidget):
             self.next.show()
             self.prev.show()
 
-    def _getCurrentImage(self):
+    @property
+    def currentImage(self):
+        """Get or set the index of the current image. If the index isn't valid
+           then a blank image is loaded."""
         return self._currentImage
 
-    def _setCurrentImage(self, num):
+    @currentImage.setter
+    def currentImage(self, num):
         while True:
             # A lot of files have corrupt picture data. I just want to
             # skip those and not have the user be any wiser.
@@ -1970,15 +1951,11 @@ class PicWidget(QWidget):
             self._image_type.setCurrentIndex(3)
         self._image_type.blockSignals(False)
         self._currentImage = num
-        self.context = str(self._contextFormat.arg(str(num + 1)).arg(str(len(self.images))))
-        self.label.setFrameStyle(QFrame.NoFrame)
+        self.context = self._contextFormat.format(str(num + 1), str(len(self.images)))
+        self.label.setFrameStyle(QFrame.Shape.NoFrame)
         self.enableButtons()
         # self.resizeEvent()
 
-    currentImage = property(_getCurrentImage, _setCurrentImage, """Get or set the index of
-    the current image. If the index isn't valid
-    then a blank image is loaded.""")
-
     def maxImage(self):
         """Shows a window with the picture fullsized."""
         if self.win.isVisible():
@@ -2019,11 +1996,13 @@ class PicWidget(QWidget):
             if not filename:
                 return
             if not self.pixmap.save(filename):
-                QMessageBox.critical(self, translate("Defaults", 'Error'),
-                                     translate("Artwork", 'Writing to <b>%1</b> failed.').arg(filename))
+                QMessageBox.critical(self,
+                                     translate('Defaults', "Error"),
+                                     translate('Artwork', "Writing to <b>{}</b> failed.").format(filename)
+                                     )
 
     def setNone(self):
-        self.label.setFrameStyle(QFrame.Box)
+        self.label.setFrameStyle(QFrame.Shape.Box)
         self.label.setPixmap(QPixmap())
         self._image_size.setText("")
         self.pixmap = None
@@ -2070,7 +2049,7 @@ class PicWidget(QWidget):
             if filename.startswith(":/"):
                 ba = QByteArray()
                 data = QBuffer(ba)
-                data.open(QIODevice.WriteOnly)
+                data.open(QIODevice.OpenModeFlag.WriteOnly)
                 image.save(data, "JPG")
                 data = str(data.data())
             else:
@@ -2085,7 +2064,7 @@ class PicWidget(QWidget):
             if image.loadFromData(data):
                 pic = {'data': data, 'height': image.height(),
                        'width': image.width(), 'size': len(data),
-                       'mime': 'image/jpeg',
+                       'mime': get_mime(data),
                        'description': "",
                        'imagetype': 3}
                 images.append(pic)
@@ -2098,7 +2077,7 @@ class PicWidget(QWidget):
             image = QImage().fromData(d)
             pic = {'data': d, 'height': image.height(),
                    'width': image.width(), 'size': len(data),
-                   'mime': 'image/jpeg',
+                   'mime': get_mime(d),
                    'description': "",
                    'imagetype': 3}
             images.append(pic)
@@ -2157,7 +2136,7 @@ class PicWin(QDialog):
         self.label.clicked.connect(self.close)
 
     def setImage(self, pixmap):
-        maxsize = QDesktopWidget().availableGeometry().size()
+        maxsize = self.screen().availableGeometry().size()
         self.label.setPixmap(pixmap)
         if hasattr(pixmap, 'size'):
             size = pixmap.size()
@@ -2175,13 +2154,14 @@ class PicWin(QDialog):
 class ProgressWin(QDialog):
     canceled = pyqtSignal(name='canceled')
 
-    def __init__(self, parent=None, maximum=100, progresstext='', showcancel=True):
+    def __init__(self, parent: Optional[QObject] = None, maximum: int = 100, progresstext: str = '',
+                 showcancel: bool = True) -> None:
         QDialog.__init__(self, parent)
         self._infunc = False
         self._cached = 0
         self.setModal(True)
         self.setWindowTitle(translate("Progress Dialog", "Please Wait..."))
-        self._format = translate("Progress Dialog", '%1%2 of %3...')
+        self._format = translate('Progress Dialog', "{}{} of {}...")
 
         self.ptext = progresstext
 
@@ -2190,7 +2170,7 @@ class ProgressWin(QDialog):
         self.pbar.setRange(0, maximum)
 
         self.label = QLabel()
-        self.label.setAlignment(Qt.AlignHCenter)
+        self.label.setAlignment(Qt.AlignmentFlag.AlignHCenter)
 
         if maximum <= 0:
             self.pbar.setTextVisible(False)
@@ -2204,7 +2184,8 @@ class ProgressWin(QDialog):
         cbox = QHBoxLayout()
         cbox.addStretch()
         cbox.addWidget(cancel)
-        cancel.setVisible(showcancel)
+        if not showcancel:
+            cancel.hide()
 
         vbox = QVBoxLayout()
         vbox.addWidget(self.label)
@@ -2229,20 +2210,20 @@ class ProgressWin(QDialog):
         if maximum <= 0:
             self._timer.start()
 
-    def setValue(self, value):
+    def setValue(self, value: int) -> None:
         if self._infunc:
             return
         self._infunc = True
         if self.ptext:
             self.pbar.setTextVisible(False)
-            self.label.setText(self._format.arg(self.ptext
-                                                ).arg(value).arg(self.pbar.maximum()))
+            self.label.setText(self._format.format(
+                self.ptext, value, self.pbar.maximum()))
         self.pbar.setValue(value)
         self._infunc = False
         if self.pbar.maximum() and value >= self.pbar.maximum():
             self.close()
 
-    def cancel(self):
+    def cancel(self) -> None:
         self.wasCanceled = True
         self.canceled.emit()
         self.close()
@@ -2252,11 +2233,10 @@ class ProgressWin(QDialog):
             self._timer.stop()
         super(ProgressWin, self).closeEvent(event)
 
-    def _value(self):
+    @property
+    def value(self) -> int:
         return self.pbar.value()
 
-    value = property(_value)
-
 
 class PuddleCombo(QWidget):
     editTextChanged = pyqtSignal(str, name='editTextChanged')
@@ -2266,10 +2246,10 @@ class PuddleCombo(QWidget):
         hbox = QHBoxLayout()
         hbox.setContentsMargins(0, 0, 0, 0)
         self.combo = QComboBox()
-        self.combo.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLength)
+        self.combo.setSizeAdjustPolicy(QComboBox.SizeAdjustPolicy.AdjustToContents)
 
         self.remove = QToolButton()
-        self.remove.setIcon(get_icon('list-remove', ':/remove.png'))
+        self.remove.setIcon(get_icon('list-remove'))
         self.remove.setToolTip(translate("Combo Box", 'Remove current item.'))
         self.remove.setIconSize(QSize(13, 13))
         self.remove.clicked.connect(self.removeCurrent)
@@ -2348,14 +2328,14 @@ class PuddleDock(QDockWidget):
 
 
 class PuddleHeader(QHeaderView):
-    def __init__(self, orientation=Qt.Horizontal, parent=None):
+    def __init__(self, orientation=Qt.Orientation.Horizontal, parent=None):
         if parent:
             super(PuddleHeader, self).__init__(orientation, parent)
         else:
             super(PuddleHeader, self).__init__()
 
         self.setSortIndicatorShown(True)
-        self.setSortIndicator(0, Qt.AscendingOrder)
+        self.setSortIndicator(0, Qt.SortOrder.AscendingOrder)
         self.setSectionsMovable(True)
         self.setSectionsClickable(True)
 
@@ -2424,20 +2404,20 @@ class PuddleThread(QThread):
     error = pyqtSignal([str, int], name='error')
     win = pyqtSignal(int, name='win')
 
-    def __init__(self, command, parent=None):
+    def __init__(self, command: Callable[[], Any], parent: Optional[QObject] = None) -> None:
         QThread.__init__(self, parent)
         self.finished.connect(self._finish)
         self.command = command
         self.retval = None
 
-    def run(self):
+    def run(self) -> None:
         # print 'thread', self.command, time.time()
         try:
             self.retval = self.command()
         except StopIteration:
             self.retval = 'STOP'
 
-    def _finish(self):
+    def _finish(self) -> None:
         if hasattr(self, 'retval'):
             self.threadfinished.emit(self.retval)
         else:
@@ -2485,15 +2465,15 @@ class ShortcutEditor(QLineEdit):
         self.setText(text)
         self.valid = valid
 
-    def _getValid(self):
+    @property
+    def valid(self):
         return self._valid
 
-    def _setValid(self, value):
+    @valid.setter
+    def valid(self, value):
         self._valid = value
         self.validityChanged.emit(value)
 
-    valid = property(_getValid, _setValid)
-
 
 if __name__ == '__main__':
     class MainWin(QDialog):
