@@ -65,28 +65,30 @@ def update_settings():
 
 
 class SettingsCheckBox(QCheckBox):
+
     def __init__(self, default=None, text=None, parent=None):
         QCheckBox.__init__(self, translate("GenSettings", text), parent)
 
         self.settingValue = default
         self._text = text
 
-    def _value(self):
-        if self.checkState() == Qt.Checked:
+    @property
+    def settingValue(self):
+        if self.checkState() == Qt.CheckState.Checked:
             return self._text, True
         else:
             return self._text, False
 
-    def _setValue(self, value):
+    @settingValue.setter
+    def settingValue(self, value):
         if value:
-            self.setCheckState(Qt.Checked)
+            self.setCheckState(Qt.CheckState.Checked)
         else:
-            self.setCheckState(Qt.Unchecked)
-
-    settingValue = property(_value, _setValue)
+            self.setCheckState(Qt.CheckState.Unchecked)
 
 
 class SettingsLineEdit(QWidget):
+
     def __init__(self, desc, default, parent=None):
         QWidget.__init__(self, parent)
         vbox = QVBoxLayout()
@@ -99,16 +101,17 @@ class SettingsLineEdit(QWidget):
         vbox.addWidget(self._text)
         self.setLayout(vbox)
 
-    def _value(self):
+    @property
+    def settingValue(self):
         return self._desc, str(self._text.text())
 
-    def _setValue(self, value):
+    @settingValue.setter
+    def settingValue(self, value):
         self._text.setText(self._desc, value)
 
-    settingValue = property(_value, _setValue)
-
 
 class GeneralSettings(QWidget):
+
     def __init__(self, controls, parent=None):
         QWidget.__init__(self, parent)
         settings = []
@@ -141,7 +144,7 @@ class GeneralSettings(QWidget):
         self._lang_combo.addItems(list(get_languages([TRANSDIR])))
 
         if lang != 'auto':
-            i = self._lang_combo.findText(lang, Qt.MatchFixedString)
+            i = self._lang_combo.findText(lang, Qt.MatchFlag.MatchFixedString)
             if i > 0:
                 self._lang_combo.setCurrentIndex(i)
 
@@ -168,7 +171,7 @@ class GeneralSettings(QWidget):
                               ['__filename,track,__dirpath', 'track, album',
                                '__filename,album,__dirpath'])
 
-        from .webdb import SortOptionEditor
+        from .mainwin.tagsources import SortOptionEditor
         win = SortOptionEditor(options, self)
         win.options.connect(self.applySortOptions)
         win.show()
@@ -198,13 +201,14 @@ class GeneralSettings(QWidget):
 
 
 class Playlist(QWidget):
+
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
 
         def inttocheck(value):
             if value:
-                return Qt.Checked
-            return Qt.Unchecked
+                return Qt.CheckState.Checked
+            return Qt.CheckState.Unchecked
 
         cparser = PuddleConfig()
 
@@ -241,8 +245,9 @@ class Playlist(QWidget):
         self.setLayout(vbox)
 
     def applySettings(self, control=None):
+
         def checktoint(checkbox):
-            if checkbox.checkState() == Qt.Checked:
+            if checkbox.checkState() == Qt.CheckState.Checked:
                 return 1
             else:
                 return 0
@@ -256,6 +261,7 @@ class Playlist(QWidget):
 
 
 class TagMappings(QWidget):
+
     def __init__(self, parent=None):
         filename = os.path.join(PuddleConfig().savedir, 'mappings')
         self._edited = deepcopy(audioinfo.mapping)
@@ -382,6 +388,7 @@ class TagMappings(QWidget):
 
 
 class Tags(QWidget):
+
     def __init__(self, parent=None, status=None):
         QWidget.__init__(self, parent)
         self._edited = False
@@ -473,25 +480,26 @@ class Tags(QWidget):
 
 
 class ListModel(QAbstractListModel):
+
     def __init__(self, options):
         QAbstractListModel.__init__(self)
         self.options = options
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if role == Qt.TextAlignmentRole:
-            if orientation == Qt.Horizontal:
-                return int(Qt.AlignLeft | Qt.AlignVCenter)
-            return int(Qt.AlignRight | Qt.AlignVCenter)
-        if role != Qt.DisplayRole:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.TextAlignmentRole:
+            if orientation == Qt.Orientation.Horizontal:
+                return int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
+            return int(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
-        if orientation == Qt.Horizontal:
+        if orientation == Qt.Orientation.Horizontal:
             return self.headerdata[section]
         return int(section + 1)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         if not index.isValid() or not (0 <= index.row() < len(self.options)):
             return None
-        if (role == Qt.DisplayRole) or (role == Qt.ToolTipRole):
+        if (role == Qt.ItemDataRole.DisplayRole) or (role == Qt.ItemDataRole.ToolTipRole):
             try:
                 return str(self.options[index.row()][0])
             except IndexError:
@@ -506,8 +514,8 @@ class ListModel(QAbstractListModel):
 
     def flags(self, index):
         if not index.isValid():
-            return Qt.ItemIsEnabled
-        return Qt.ItemFlags(QAbstractListModel.flags(self, index))
+            return Qt.ItemFlag.ItemIsEnabled
+        return Qt.ItemFlag(QAbstractListModel.flags(self, index))
 
 
 class SettingsList(QListView):
@@ -524,13 +532,15 @@ class SettingsList(QListView):
 
 
 class StatusWidgetItem(QTableWidgetItem):
+
     def __init__(self, text, color):
         QTableWidgetItem.__init__(self, text)
         self.setBackground(QBrush(color))
-        self.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
+        self.setFlags(Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled)
 
 
 class ColorEdit(QWidget):
+
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         cparser = PuddleConfig()
@@ -544,7 +554,7 @@ class ColorEdit(QWidget):
             *cparser.get('table', key, default, True))
 
         preview = get_color('preview_color', [192, 255, 192])
-        selection_default = QPalette().color(QPalette.Mid).getRgb()[:-1]
+        selection_default = QPalette().color(QPalette.ColorRole.Mid).getRgb()[:-1]
 
         selection = get_color('selected_color', selection_default)
 
@@ -554,7 +564,7 @@ class ColorEdit(QWidget):
         label = QLabel(text)
 
         self.listbox = QTableWidget(0, 1, self)
-        self.listbox.setEditTriggers(QAbstractItemView.NoEditTriggers)
+        self.listbox.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
         header = self.listbox.horizontalHeader()
         self.listbox.setSortingEnabled(False)
         header.setVisible(True)
@@ -657,7 +667,7 @@ class SettingsDialog(QDialog):
         self.listbox.setModel(self.model)
 
         self.stack = QStackedWidget()
-        self.stack.setFrameStyle(QFrame.StyledPanel)
+        self.stack.setFrameStyle(QFrame.Shape.StyledPanel)
 
         self.grid = QGridLayout()
         self.grid.addWidget(self.listbox)
@@ -672,7 +682,7 @@ class SettingsDialog(QDialog):
         index = self.model.index(0, 0)
         selection.select(index, index)
         self.listbox.setSelectionModel(self.selectionModel)
-        self.selectionModel.select(selection, QItemSelectionModel.Select)
+        self.selectionModel.select(selection, QItemSelectionModel.SelectionFlag.Select)
 
         self.okbuttons = OKCancel()
         self.okbuttons.okButton.setDefault(True)
@@ -697,7 +707,9 @@ class SettingsDialog(QDialog):
                 z[1].applySettings(z[2])
             except SettingsError as e:
                 QMessageBox.warning(self, 'puddletag',
-                                    translate('Settings', 'An error occurred while saving the settings of <b>%1</b>: %2').arg(z[0]).arg(str(e)))
+                                    translate('Settings',
+                                              "An error occurred while saving the settings of <b>{}</b>: {}"
+                                              ).format(z[0], str(e)))
                 return
         self.close()
 
