@@ -1,23 +1,26 @@
 import logging
 import os
+import platform
 import sys
+import urllib.parse
 from functools import partial
 
 from PyQt5.QtCore import QDir, QSettings, QUrl, pyqtRemoveInputHook, pyqtSignal
-from PyQt5.QtGui import QDesktopServices, QIcon
+from PyQt5.QtGui import QDesktopServices
 from PyQt5.QtWidgets import QAction, QApplication, QFileDialog, QFrame, QLabel, QMainWindow, QMenu, QMessageBox, QSplitter, QVBoxLayout, QWidget
 
 from . import loadshortcuts as ls
 from . import m3u, genres
 from . import mainwin
 from . import tagmodel
-from . import webdb
+from . import version_string, changeset
 from .mainwin import funcs as mainfuncs
 from .masstag import dialogs
 from .puddleobjects import (PuddleConfig, PuddleDock, winsettings,
                             progress, PuddleStatus, errormsg, dircmp, get_icon)
 from .puddlesettings import SettingsDialog, load_gen_settings, update_settings
 from .tagmodel import TagTable
+from .about import versions
 
 from . import audioinfo
 from .util import rename_error_msg
@@ -66,7 +69,7 @@ def create_tool_windows(parent, extra=None):
     cparser.filename = ls.menu_path
     widgets = (mainwin.tagpanel, mainwin.artwork,
                mainwin.dirview, mainwin.patterncombo, mainwin.filterwin,
-               webdb, mainwin.storedtags, mainwin.logdialog,
+               mainwin.tagsources, mainwin.storedtags, mainwin.logdialog,
                dialogs)
 
     controls = [z.control for z in widgets]
@@ -194,6 +197,105 @@ def connect_action_shortcuts(actions):
             action.setShortcut(shortcut)
 
 
+def get_os():
+    """Returns a string describing the underlying operating system."""
+
+    def osrelease_dict_to_str(osrel):
+        """Returns a pretty string from the given os-release dict."""
+        # First try: The PRETTY_NAME usually already includes a version
+        if 'PRETTY_NAME' in osrel:
+            return osrel['PRETTY_NAME']
+
+        # Second try: Combine name and version
+        name = next((osrel[k] for k in ['NAME', 'ID'] if k in osrel), 'Linux')
+        version = next((osrel[k] for k in ['VERSION', 'VERSION_ID', 'VERSION_CODENAME'] if k in osrel), '')
+        return '%s %s' % (name, version)
+
+    def read_osrelease():
+        """Naive re-implementation of freedesktop_os_release."""
+        for filepath in ['/etc/os-release', '/usr/lib/os-release']:
+            osrelease_dict = {}
+            try:
+                with open(filepath, 'r', -1, 'utf-8') as file:
+                    for line in file:
+                        line = line.strip()
+                        if line.startswith('#'):
+                            continue
+                        name, value = line.split('=', 1)
+                        osrelease_dict[name.strip()] = value.strip('"\'').strip()
+            except OSError:
+                pass
+            else:
+                return osrelease_dict
+        raise OSError('No file to read found')
+
+    def get_linux_name():
+        """Returns the name of the linux distribution. Hopefully."""
+        # First try: The optional distro package (should always work if available)
+        try:
+            import distro
+        except ImportError:
+            pass
+        else:
+            return distro.name(True)
+
+        # Second try: the new function in platform (py v3.10+ only)
+        if hasattr(platform, 'freedesktop_os_release'):
+            try:
+                return osrelease_dict_to_str(platform.freedesktop_os_release())
+            except OSError:
+                pass
+
+        # Third try: the old function in platform (until py v3.7 only)
+        if hasattr(platform, 'linux_distribution'):
+            linux_dist = platform.linux_distribution()
+            if any((x for x in linux_dist if x)):
+                return '%s %s' % (linux_dist[0], linux_dist[1])
+
+        # Fourth try: read the os-release file directly (to fill the gap btw. 3.7 and 3.10)
+        try:
+            return osrelease_dict_to_str(read_osrelease())
+        except OSError:
+            pass
+
+        # We tried hard, but it wasn't enough.
+        return None
+
+    if 'Linux' == platform.system():
+        result = get_linux_name()
+        if result:
+            return result
+        # else fall through to the fallback
+    if 'Windows' == platform.system():
+        win_info = platform.win32_ver()
+        return 'Windows %s' % (win_info[0])
+    if 'Darwin' == platform.system():
+        return 'macOS %s' % (platform.mac_ver()[0])
+
+    # In case of emergency, hope for the best
+    return '%s %s %s' % (platform.system(), platform.release(), platform.version())
+
+
+def create_bug_report_issue():
+    """Create a new, pre-filled bug report as github issue."""
+    puddletag_version = version_string
+    if changeset:
+        puddletag_version = f'{puddletag_version} ({changeset})'
+    version_list = [
+        ('Puddletag', puddletag_version),
+        ('OS', get_os().strip()),
+        *versions().items()
+    ]
+
+    params = urllib.parse.urlencode({
+        'template': 'bug_report.yaml',
+        'labels': 'bug',
+        'system': '\n'.join(map(lambda x: f'{x[0]}: {x[1]}', version_list)),
+    })
+    url = f'https://github.com/puddletag/puddletag/issues/new?{params}'
+    QDesktopServices.openUrl(QUrl(url))
+
+
 def help_menu(parent):
     menu = QMenu(translate("Menus", 'Help'), parent)
     open_url = lambda url: QDesktopServices.openUrl(QUrl(url))
@@ -208,11 +310,10 @@ def help_menu(parent):
     connect(forum_link,
             lambda: open_url('https://github.com/puddletag/puddletag'))
 
-    issue_link = QAction(translate("Menus", '&Bug tracker'), parent)
-    connect(issue_link,
-            lambda: open_url('https://github.com/puddletag/puddletag/issues'))
+    issue_link = QAction(translate("Menus", '&Report a problem'), parent)
+    connect(issue_link, create_bug_report_issue)
 
-    about_icon = get_icon('help-about', QIcon())
+    about_icon = get_icon('help-about')
     about = QAction(about_icon,
                     translate("Menus", 'About puddletag'), parent)
     connect(about, partial(mainfuncs.show_about, parent))
@@ -251,6 +352,7 @@ class PreviewLabel(QLabel):
         self._enabled = not self._enabled
         self.valueChanged.emit(self._enabled)
 
+
 def _openFilesFilterFilename(filename):
     filename = os.path.abspath(filename)
     if isinstance(filename, str):
@@ -258,6 +360,11 @@ def _openFilesFilterFilename(filename):
     return filename
 
 
+# set in MainWin.__init__
+add_shortcuts = None
+remove_shortcuts = None
+
+
 class MainWin(QMainWindow):
     loadFiles = pyqtSignal(object, object, object, object, object, name='loadFiles')
     always = pyqtSignal(bool, name='always')
@@ -441,20 +548,21 @@ class MainWin(QMainWindow):
             initial = initial.decode('utf8', 'replace')
 
         if len(dirs) > 1:
-            self.setWindowTitle(translate("Main Window", 'puddletag: %1 + others').arg(initial))
+            self.setWindowTitle(translate('Main Window', "puddletag: {} + others").format(initial))
         else:
-            self.setWindowTitle(translate("Main Window", 'puddletag: %1').arg(initial))
+            self.setWindowTitle(translate('Main Window', "puddletag: {}").format(initial))
 
         self._lastdir = dirs
 
     def _getDir(self):
         dirname = self._lastdir[0] if self._lastdir else QDir.homePath()
         filedlg = QFileDialog()
-        filedlg.setFileMode(filedlg.DirectoryOnly)
-        # not supported in PyQt5
-        # filedlg.setResolveSymlinks(False) 
-        filename = str(filedlg.getExistingDirectory(self,
-                                                    translate("Main Window", 'Import directory...'), dirname, QFileDialog.ShowDirsOnly|QFileDialog.DontUseNativeDialog))
+        filedlg.setFileMode(QFileDialog.FileMode.Directory)
+        filedlg.setOption(QFileDialog.Option.ShowDirsOnly)
+        filename = str(QFileDialog.getExistingDirectory(self,
+                                                        translate("Main Window", 'Import directory...'),
+                                                        dirname,
+                                                        QFileDialog.Option.ShowDirsOnly | QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontResolveSymlinks))
         return filename
 
     def appendDir(self, filename=None):
@@ -486,7 +594,7 @@ class MainWin(QMainWindow):
                     control.saveSettings()
 
         cparser = PuddleConfig()
-        settings = QSettings(constants.QT_CONFIG, QSettings.IniFormat)
+        settings = QSettings(constants.QT_CONFIG, QSettings.Format.IniFormat)
         if self._lastdir:
             cparser.set('main', 'lastfolder', self._lastdir[0])
         cparser.set("main", "maximized", self.isMaximized())
@@ -500,7 +608,7 @@ class MainWin(QMainWindow):
     def createStatusBar(self):
         statusbar = self.statusBar()
         statuslabel = QLabel()
-        statuslabel.setFrameStyle(QFrame.NoFrame)
+        statuslabel.setFrameStyle(QFrame.Shape.NoFrame)
         statusbar.addPermanentWidget(statuslabel, 1)
         self._totalstats = QLabel('00 (00:00:00 | 00 MB)')
         self._selectedstats = QLabel('00 (00:00:00 | 00 MB)')
@@ -529,7 +637,7 @@ class MainWin(QMainWindow):
     def loadPlayList(self):
         dirname = self._lastdir[0] if self._lastdir else QDir.homePath()
         selectedFile = QFileDialog.getOpenFileName(self,
-                                                   translate("Playlist", translate("Playlist", 'Select m3u file...')), )
+                                                   translate("Playlist", translate("Playlist", 'Select m3u file...')),)
         filename = selectedFile[0]
         if not filename:
             return
@@ -539,10 +647,20 @@ class MainWin(QMainWindow):
         except (OSError, IOError) as e:
             QMessageBox.information(self._table,
                                     translate("Defaults", 'Error'),
-                                    translate("Playlist", 'An error occured while reading <b>%1</b> (%2)').arg(filename).arg(e.strerror))
+                                    translate('Playlist',
+                                              "An error occured while reading <b>{}</b> ({})"
+                                              ).format(filename, e.strerror)
+                                    )
+        except (UnicodeError) as e:
+            QMessageBox.information(self._table,
+                                    translate("Defaults", 'Error'),
+                                    translate("Playlist", 'The playlist is not encoded in UTF-8'))
         except Exception as e:
             QMessageBox.information(self._table, translate("Defaults", 'Error'),
-                                    translate("Playlist", 'An error occured while reading <b>%1</b> (%2)').arg(filename).arg(str(e)))
+                                    translate('Playlist',
+                                              "An error occured while reading <b>{}</b> ({})"
+                                              ).format(filename, str(e))
+                                    )
 
     def openDir(self, filename=None, append=False):
         """Opens a folder. If filename != None, then
@@ -599,7 +717,7 @@ class MainWin(QMainWindow):
         connect_actions(scts, PuddleDock._controls)
 
         cparser = PuddleConfig()
-        settings = QSettings(constants.QT_CONFIG, QSettings.IniFormat)
+        settings = QSettings(constants.QT_CONFIG, QSettings.Format.IniFormat)
 
         gensettings = {}
         controls = list(PuddleDock._controls.values())
@@ -784,6 +902,7 @@ class MainWin(QMainWindow):
 
                 model.updateTable(failed_rows)
             return fin()
+
         return func, finished, rows
 
     def writeTags(self, tagiter, rows=None, previews=None):
@@ -899,9 +1018,9 @@ class MainWin(QMainWindow):
                     setRowData(row, {tag: filename}, True, True)
                     yield None
                 except EnvironmentError as e:
-                    m = translate("Dir Renaming",
-                                  'An error occured while renaming <b>%1</b> to ' \
-                                  '<b>%2</b>. (%3)').arg(audio[PATH]).arg(filename).arg(e.strerror)
+                    m = translate('Dir Renaming',
+                                  "An error occured while renaming <b>{}</b> to <b>{}</b>. ({})"
+                                  ).format(audio[PATH], filename, e.strerror)
                     if row == rows[-1]:
                         yield m, 1
                     else:
@@ -923,7 +1042,9 @@ class MainWin(QMainWindow):
                 if self._lastdir and olddir in self._lastdir:
                     self._lastdir[self._lastdir.index(olddir)] = newdir
             except (IOError, OSError) as detail:
-                msg = translate("Dir Renaming", "I couldn't rename: <i>%1</i> to <b>%2</b> (%3)").arg(olddir).arg(newdir).arg(detail.strerror)
+                msg = translate('Dir Renaming',
+                                "I couldn't rename: <i>{}</i> to <b>{}</b> ({})"
+                                ).format(olddir, newdir, detail.strerror)
                 if index == len(dirs) - 1:
                     dirlen = 1
                 else:
